# frozen_string_literal: true

# checker Api::Dmo::HermesController avant d'ajouter des méthodes
module Hermes
  module Api
    module Dmo
      class EntityValidationsController < Hermes::Api::Dmo::ApplicationController
        VALID_OPERATORS = %w[
          $in
          $nin
          $ne
          $gt
          $gte
          $lt
          $lte
          $regex
        ].freeze

        ACCEPTABLE_PARAMS = %w[
          ids
          applicant_id
          identification_number
          salutation
          name
          surname
          birthdate
          validation
          contract_start_date
          contract_end_date
          active
        ].freeze

        REGEX_PARAMS = %w[
        ].freeze

        MUTATE_PARAMS = %i[
          applicant_id
          identification_number
          civility
          first_name
          last_name
          birth_date
          validation
          contract_start_date
          contract_end_date
        ].freeze
        mutate_params_only(*MUTATE_PARAMS)

        before_action :set_entity

        def index
          @query_precision ||= {}
          @query_precision[:entity_id] = @entity.id
          super
        end

        def create
          user_id = current_user.id
          param_user = params&.fetch(:data, nil)&.fetch(:attributes, nil)&.fetch(:user_id, nil)
          user_id = param_user if @user.has_role?('Administrator') && param_user
          @instance = @model.new(mutate_valid_params.merge(user_id: user_id, entity_id: @entity.id))
          return respond_with_errors(@instance) unless @instance.save

          render(jsonapi: @instance, serializer: @serializer, status: :created)
        end

        private

        def set_entity
          @entity = Hermes::Entity.find(params[:entity_id]) if params[:entity_id].present?
          @entity ||= Hermes::Entity.find_by(_type: "Hermes::Entity", user_id: current_user.id)
          return render_json_error(:not_found, :entity) unless @entity
        end
      end
    end
  end
end
