module Hermes
    class EntityRequestAccount
        include Hermes::Concerns::CommonFieldsConcern
        field :amount, type: Integer
        field :origin_entity_id, type: String
        field :entity_id, type: String
        field :request_rented, type: Boolean, default: false

        validates_presence_of :amount, :origin_entity_id, :entity_id
        belongs_to :housing_request
        after_create :after_create_actions

        writer_auth_roles 'collaborateur entreprise'
        reader_auth_roles 'collaborateur offre logement'


        common_indexes *%i[
          amount
          origin_entity_id
          entity_id
          request_rented
        ]

        def after_create_actions
            entity = Hermes::Entity.find_by(id: self.entity_id)
            points = self.amount
            available_points = entity.available_points
            if points <= available_points
                final_points = available_points - points
                entity.available_points = final_points
                entity.save
            else
                entity.available_points = 0 
                entity.save
            end 
        end 
    end 
end 
