# frozen_string_literal: true

# Return Hermes::HousingRequest attributes from parsed xml

module Hermes
  module BoClient
    module Mappings
      class HousingRequestReverseMap        
        def initialize(xml)
          @xml = xml
        end

        def build_housing_request
          demande_logement = @xml["interfaceNuu"]["demande"]["demandeLogement"] #["personnePhysique"]["demandeur"]["mel"]
          personnePhysique = demande_logement["personnePhysique"]
          demandeur = personnePhysique["demandeur"]
          attributes = {
            ec_law_ressources: demande_logement["ressourcesLoiEC"],
            lessor_siren: personnePhysique["logementActuel"]&.fetch("compSIRENOrgBailleur", nil),
            electronical_renewal: demande_logement["renouvellementElec"],
            widen_request: demande_logement["logementRecherche"]["demandeElargie"],
            lessor_name: personnePhysique['logementActuel']['compNomOrgBailleur'],
            work_contract_type: [
              demandeur["situationProfessionnelle"]["typeContratTravail"]&.fetch("code", nil)
            ],
            dalo: @xml["interfaceNuu"]["demande"]&.fetch("informationDalo", nil)&.fetch("statutDalo", nil)&.fetch("code", nil) == "ACTIF" ? true : false,
            with_car_park: demande_logement["logementRecherche"]["avecParking"],
            mutation: !!demande_logement['renouvellement'],
            seeking_first_floor: demande_logement["logementRecherche"]["rezDeChaussee"],
            seeking_elevator: demande_logement["logementRecherche"]["sansAscenseur"],
            seeked_max_rent: demande_logement["logementRecherche"]["montantLoyer"].to_i,
            precisions: demande_logement["precision"],
            seeked_housing_categories: [
              demande_logement["logementRecherche"]["categorieLogementRecherche"]&.fetch("code", nil)
            ],
            is_house_owner: personnePhysique["logementActuel"]["proprietaire"],
            
            requester_firstname: demandeur["prenom"],
            requester_lastname: demandeur["nom"],
            
            requester_email: demandeur["mel"],
            # created_at: demande_logement["dateCreationDemande"],
            sne_created_at: demande_logement["dateCreationDemande"],
            radiation_date: demande_logement["radiation"]&.fetch("dateRadiation", nil),
            radiation_motive: demande_logement["radiation"]&.fetch("motifRadiation", nil)&.fetch("code", nil),
            
          }
          attributes[:forcing_duplicate] = (demande_logement["forcerDoublon"] != "false") if demande_logement["forcerDoublon"]
          
          logementActuel = personnePhysique["logementActuel"] || {}
          if logementActuel
            modeleLogement = logementActuel["listeModeleLogement"]&.fetch("modeleLogement", nil)
            attributes[:housing_type_list] = modeleLogement.is_a?(Array) ? [] : [modeleLogement&.fetch("code", nil)]
            if modeleLogement.is_a?(Array)
              attributes[:housing_type_list] = []
              modeleLogement.each_with_index do |val|
                attributes[:housing_type_list].push (val&.fetch("code", nil))
              end
            end
          end
          if personnePhysique["personneANaitre"]
            attributes[:unborn_children] = [{"anticipated_birth" => personnePhysique["personneANaitre"]["dateNaissancePrevue"], "nb_child_birth" => personnePhysique["personneANaitre"]["nombre"]}]
          end
          
          if (demande_logement["principauxMotifDemande"]["motifDemande"].is_a? Hash)
            attributes[:motivation1] = demande_logement["principauxMotifDemande"]["motifDemande"]&.fetch("code", nil)
          else
            attributes[:motivation1] = demande_logement["principauxMotifDemande"]["motifDemande"]&.fetch(0, {})&.fetch("code", nil)
            attributes[:motivation2] = demande_logement["principauxMotifDemande"]["motifDemande"]&.fetch(1, {})&.fetch("code", nil)
            attributes[:motivation3] = demande_logement["principauxMotifDemande"]["motifDemande"]&.fetch(2, {})&.fetch("code", nil)
          end
          attributes[:seeked_types] = []
          typeLogement = demande_logement["logementRecherche"]["listeTypeLogement"]["typeLogement"]
          typeLogement = typeLogement.is_a?(Array) ? typeLogement : [typeLogement]
          typeLogement&.each do |type|
            attributes[:seeked_types].push(type&.fetch("code", nil).to_s)
          end
          
          attributes[:dependents] = []
          demandeur["listeRessourceRecuePersACharge"] = demandeur["listeRessourceRecuePersACharge"].is_a?(Array) ? demandeur["listeRessourceRecuePersACharge"] : [demandeur["listeRessourceRecuePersACharge"]]
          demandeur["listeRessourceRecuePersACharge"]&.each_with_index do |persACharge, i|
            
            if persACharge && persACharge["detailRessource"]
              attributes[:dependents][i] ||= {}
              attributes[:dependents][i]["monthly_incomes"] ||= {}
              ressources = persACharge["detailRessource"].is_a?(Array) ? persACharge["detailRessource"] : [persACharge["detailRessource"]]
              ressources.each do |ressource|
                attributes[:dependents][i]["monthly_incomes"][ressource&.fetch("code", nil).to_s] = ressource["montant"].to_i
              end
            
            end
          end
          
          demandeur["listeRessourceVerseePersACharge"] = demandeur["listeRessourceVerseePersACharge"].is_a?(Array) ? demandeur["listeRessourceVerseePersACharge"] : [demandeur["listeRessourceVerseePersACharge"]]
          demandeur["listeRessourceVerseePersACharge"]&.each_with_index do |persACharge, i|
            if persACharge && persACharge["detailRessource"]
              attributes[:dependents][i] ||= {}
              attributes[:dependents][i]["monthly_charges"] ||= {}
              attributes[:dependents][i]["monthly_charges"][persACharge["detailRessource"]["ressource"]&.fetch("code", nil).to_s] = persACharge["detailRessource"]["montant"].to_i
            end
          end
          if personnePhysique["listePersonneACharge"] && personnePhysique["listePersonneACharge"]["personneACharge"]
            personnePhysique["listePersonneACharge"]["personneACharge"] = personnePhysique["listePersonneACharge"]["personneACharge"].is_a?(Array) ? personnePhysique["listePersonneACharge"]["personneACharge"] : [personnePhysique["listePersonneACharge"]["personneACharge"]]
            
            personnePhysique["listePersonneACharge"]["personneACharge"]&.each_with_index do |persACharge, i|
              if persACharge
                
                attributes[:dependents][i] ||= {}
                attributes[:dependents][i]["last_name"] = persACharge["nom"]
                attributes[:dependents][i]["first_name"] = persACharge["prenom"]
                attributes[:dependents][i]["birth_date"] = persACharge["dateNaissance"]
                attributes[:dependents][i]["rel_type"] = persACharge["lienParente"]&.fetch("code", nil)
                attributes[:dependents][i]["sex"] = persACharge["sexe"]&.fetch("code", nil)
              
              end
            end
          end
          dependents_index = attributes[:dependents].length 
          if (personnePhysique["listePersonneEnGarde"])
            personnePhysique["listePersonneEnGarde"]["personneEnGarde"] = personnePhysique["listePersonneEnGarde"]["personneEnGarde"].is_a?(Array) ? personnePhysique["listePersonneEnGarde"]["personneEnGarde"] : [personnePhysique["listePersonneEnGarde"]["personneEnGarde"]]
            personnePhysique["listePersonneEnGarde"]["personneEnGarde"]&.each_with_index do |persACharge, i|
              if persACharge
                attributes[:dependents][i + dependents_index] ||= {}
                attributes[:dependents][i + dependents_index]["last_name"] ||= persACharge["nom"]
                attributes[:dependents][i + dependents_index]["first_name"] ||= persACharge["prenom"]
                attributes[:dependents][i + dependents_index]["birth_date"] ||= persACharge["dateNaissance"]
                attributes[:dependents][i + dependents_index]["coparentality"] = persACharge&.fetch("coParentalite")&.fetch("code", nil)
                attributes[:dependents][i + dependents_index]["sex"] ||= persACharge["sexe"]&.fetch("code", nil)
              end
            end
            
          end
          
          
          attributes[:postal_details] = {}
          adressePostale = demandeur["adressePostale"] || {}
          attributes[:postal_details]['etranger'] = adressePostale["etranger"]
          attributes[:postal_details]['appartment_number'] = adressePostale["appartement"]
          attributes[:postal_details]['appartment_floor'] = adressePostale["etage"]
          attributes[:postal_details]['appartment_staircase'] = adressePostale["escalier"]
          attributes[:postal_details]['appartment_building'] = adressePostale["batiment"]
          attributes[:postal_details]['address_add_address'] = adressePostale["compAdresse"]
          attributes[:postal_details]['address_number'] = adressePostale["numero"]
          attributes[:postal_details]['address_street'] = adressePostale["voie"]
          attributes[:postal_details]['appartment_address_place'] = adressePostale["lieuDit"]
          attributes[:postal_details]['postal_code'] = adressePostale["codePostal"]
          attributes[:postal_details]['current_address_postal_code'] = adressePostale["codePostalEtranger"]
          attributes[:postal_details]['foreign_city'] = adressePostale["communeEtranger"]
          attributes[:postal_details]['foreign_country'] = adressePostale["pays"]
          attributes[:postal_details]['houser_name'] = adressePostale["nomHebergeant"]
          attributes[:postal_details]['address_insee_code'] = adressePostale["commune"]&.fetch("code", nil)
          
          attributes[:seeked_localisation_list] = []
          localisationSouhaite = demande_logement["logementRecherche"]["listeLocalisationSouhaite"]["localisationSouhaite"]
          localisationSouhaite = localisationSouhaite.is_a?(Array) ? localisationSouhaite : [localisationSouhaite]
          localisationSouhaite.each_with_index do |loc, i|
            if (loc)
              res = {}
              res["code"] = loc["commune"]["code"]
              res["codesPostaux"] = [loc["codePostal"]]
              res["seeked_epci_first_quartil"] = loc["seuilEPCIPremierQuartile"]
              res["seeked_quartil_belongings"] = loc["quartileAppartenance"]
              attributes[:seeked_localisation_list].push res
            end
          end
          
          if demande_logement["listeHandicap"]
            if demande_logement["listeHandicap"].is_a?(Hash)
              demande_logement["listeHandicap"] = demande_logement["listeHandicap"]["handicap"].is_a?(Array) ? demande_logement["listeHandicap"]["handicap"] : [demande_logement["listeHandicap"]["handicap"]]
            end
            attributes[:disability_details] = [{}]
            demande_logement["listeHandicap"].each_with_index do |handicap, i|
              
              handicap = handicap
              
              attributes[:disability_details][i] ||= {}
              attributes[:disability_details][i] = {"disability_states": handicap["etatHandicap"]&.fetch("code", nil),
                                                    "walking_capacities": handicap["capaciteMarcheHandicap"]&.fetch("code", nil),
              
              
                                                    "birth_date": handicap["dateNaissance"],
                                                    "accompanied": handicap["accompagne"] == "true",
                                                    "physical_disability": handicap["handicapMoteur"] == "true",
                                                    "sensory_disability": handicap["handicapSensoriel"] == "true",
                                                    "other_disability": handicap["handicapAutre"] == "true",
                                                    "special_need": handicap["compAutreNature"],
                                                    "other_need": handicap["besoinAutre"],
              }
              if handicap["referent"]
                attributes[:disability_details][i]["disability_referent"] = {
                  "recognized_disability": (handicap["referent"]["handicapReconnu"] == "true"),
                  "near_home": (handicap["referent"]["procheDomicile"] == "true"),
                  "name": handicap["referent"]["nom"],
                  "address": handicap["referent"]["adresse"],
                  "phone": handicap["referent"]["tel"],
                  "email": handicap["referent"]["mel"],
                }
              end
              if handicap["listeNatureHandicapSensoriel"]
                attributes[:disability_details][i]["sensory_disability_kinds"] = extract_codes(handicap, 'listeNatureHandicapSensoriel')
              end
              if handicap["listeNatureHandicapMoteur"]
                attributes[:disability_details][i]["physical_disability_kinds"] = extract_codes(handicap, 'listeNatureHandicapMoteur')
              end
              if handicap["listeAideTechniqueHandicap"]
                attributes[:disability_details][i]["disability_technical_helps"] = extract_codes(handicap, 'listeAideTechniqueHandicap')
              end
              if handicap["listeEquipementHandicap"]
                attributes[:disability_details][i]["disability_equipments"] = extract_codes(handicap, 'listeEquipementHandicap')
              end
            end
          end
          attributes
        end

        def extract_codes(handicap, attribute_name)
          return [] unless handicap[attribute_name]
    
          extrated_array = if handicap[attribute_name].is_a?(Array)
            if handicap[attribute_name][0] && handicap[attribute_name][0].is_a?
              handicap[attribute_name][0]
            else
              handicap[attribute_name]
            end
          elsif handicap[attribute_name].is_a?(Hash)
            [handicap[attribute_name]]
          end
    
          extrated_array.map{|o| o.fetch("code", nil)}
        end 
      end
    end
  end
end
